@@ -1,6 +1,6 @@
 /*
  * Convert ASCII to PostScript.
- * Copyright (c) 1995-1998 Markku Rossi.
+ * Copyright (c) 1995-2002 Markku Rossi.
  *
  * Author: Markku Rossi <mtr@iki.fi>
  */
@@ -78,6 +78,8 @@ typedef enum
   tFONT,
   tCOLOR,
   tBGCOLOR,
+  tSAVEX,
+  tLOADX,
   tPS
 } TokenType;
 
@@ -95,6 +97,8 @@ typedef enum
   ESC_SHADE,
   ESC_BGGRAY,
   ESC_ESCAPE,
+  ESC_SAVEX,
+  ESC_LOADX,
   ESC_PS
 } SpecialEscape;
 
@@ -216,11 +220,12 @@ static void handle_two_side_options ();
  * Global variables.
  */
 
-unsigned int current_pagenum;	/* The number of the current page. */
+unsigned int current_pagenum = 0; /* The number of the current page. */
 unsigned int total_pages_in_file;
 unsigned int input_filenum = 0;
 unsigned int current_file_linenum;
-char fname[1024];		/* The name of the current input file. */
+int first_pagenum_for_file;
+char *fname = NULL;		/* The name of the current input file. */
 
 
 /*
@@ -262,6 +267,10 @@ static Color user_bgcolor;
 /* The last linenumber printed by print_line_number(). */
 static unsigned int print_line_number_last;
 
+/* Registers to store X-coordinates with the ^@savex{} escape.
+   Initially these are uninitialized. */
+static double xstore[256];
+
 /*
  * Global functions.
  */
@@ -534,7 +543,7 @@ dump_ps_trailer ()
 
 
 void
-process_file (char *fname_arg, InputStream *is)
+process_file (char *fname_arg, InputStream *is, int is_toc)
 {
   int col;
   double x, y;
@@ -552,12 +561,16 @@ process_file (char *fname_arg, InputStream *is)
   unsigned int current_slice = 1;
   int last_wrapped_line = -1;
   int last_spaced_file_linenum = -1;
+  int save_current_pagenum;
+  int toc_pagenum = 0;
 
   /* Save filename. */
-  strcpy (fname, fname_arg);
+  xfree (fname);
+  fname = xstrdup (fname_arg);
 
   /* Init page number and line counters. */
-  current_pagenum = 0;
+  if (!continuous_page_numbers)
+    current_pagenum = 0;
   total_pages_in_file = 0;
   current_file_linenum = start_line_number;
 
@@ -597,6 +610,9 @@ process_file (char *fname_arg, InputStream *is)
   linewidth = d_output_w / num_columns - 2 * d_output_x_margin
     - line_indent;
 
+  /* Save the current running page number for possible toc usage. */
+  first_pagenum_for_file = total_pages + 1;
+
   /*
    * Divert our output to a temp file.  We will re-process it
    * afterwards to patch, for example, the number of pages in the
@@ -694,8 +710,18 @@ process_file (char *fname_arg, InputStream *is)
 		  if (do_print)
 		    total_pages++;
 
+		  if (is_toc)
+		    {
+		      save_current_pagenum = current_pagenum;
+		      toc_pagenum--;
+		      current_pagenum = toc_pagenum;
+		    }
+
 		  dump_ps_page_header (fname, 0);
 		  page_clear = 0;
+
+		  if (is_toc)
+		    current_pagenum = save_current_pagenum;
 		}
 
 	      /* Print line highlight. */
@@ -1027,7 +1053,8 @@ large for page\n"),
 		  break;
 
 		case tSETFILENAME:
-		  strcpy (fname, token.u.filename);
+		  xfree (fname);
+		  fname = xstrdup (token.u.filename);
 		  break;
 
 		case tSETPAGENUMBER:
@@ -1039,6 +1066,14 @@ large for page\n"),
 		    goto end_of_page;
 		  break;
 
+		case tSAVEX:
+		  xstore[(unsigned char) token.u.i] = x;
+		  break;
+
+		case tLOADX:
+		  x = xstore[(unsigned char) token.u.i];
+		  break;
+
 		case tPS:
 		  OUTPUT ((cofp, "%g %g M\n%s\n", x, y, token.u.str));
 		  xfree (token.u.str);
@@ -1073,10 +1108,16 @@ large for page\n"),
   if (toc)
     {
       char *cp;
+      int save_total_pages = total_pages;
+
+      /* use first pagenum in file for toc */
+      total_pages = first_pagenum_for_file;
 
       cp = format_user_string ("TOC", toc_fmt_string);
       fprintf (toc_fp, "%s\n", cp);
       xfree (cp);
+
+      total_pages = save_total_pages;
     }
 }
 
@@ -1144,6 +1185,8 @@ static struct
     {"shade",		ESC_SHADE},
     {"bggray",		ESC_BGGRAY},
     {"escape",		ESC_ESCAPE},
+    {"savex",		ESC_SAVEX},
+    {"loadx",		ESC_LOADX},
     {NULL, 0},
   };
 
@@ -1518,6 +1561,16 @@ read_special_escape (InputStream *is, Token *token)
 	  token->type = tNEWPAGE;
 	  break;
 
+	case ESC_SAVEX:
+	  token->type = tSAVEX;
+	  token->u.i = atoi (buf);
+	  break;
+
+	case ESC_LOADX:
+	  token->type = tLOADX;
+	  token->u.i = atoi (buf);
+	  break;
+
 	case ESC_PS:
 	  token->u.str = xstrdup (buf);
 	  token->type = tPS;
@@ -2069,8 +2122,16 @@ dump_ps_page_header (char *fname, int empty)
       OUTPUT ((cofp, "%% N-up sub-page %d/%d\n", nup_subpage + 1, nup));
       if (landscape)
 	{
-	  xm = nup_subpage / nup_rows;
-	  ym = nup_subpage % nup_rows;
+	  if (nup_columnwise)
+	    {
+	      xm = nup_subpage % nup_columns;
+	      ym = nup_subpage / nup_columns;
+	    }
+	  else
+	    {
+	      xm = nup_subpage / nup_rows;
+	      ym = nup_subpage % nup_rows;
+	    }
 
 	  OUTPUT ((cofp, "%d %d translate\n",
 		   xm * (nup_width + nup_xpad),
@@ -2078,11 +2139,20 @@ dump_ps_page_header (char *fname, int empty)
 	}
       else
 	{
-	  xm = nup_subpage % nup_columns;
-	  ym = nup_subpage / nup_columns;
+	  if (nup_columnwise)
+	    {
+	      xm = nup_subpage / nup_rows;
+	      ym = nup_subpage % nup_rows;
+	    }
+	  else
+	    {
+	      xm = nup_subpage % nup_columns;
+	      ym = nup_subpage / nup_columns;
+	    }
+
 	  OUTPUT ((cofp, "%d %d translate\n",
 		   xm * (nup_width + nup_xpad),
-		   -(ym * (nup_height + nup_ypad) + nup_height)));
+		   -((int) (ym * (nup_height + nup_ypad) + nup_height))));
 	}
       OUTPUT ((cofp, "%g dup scale\n", nup_scale));
 
@@ -2177,6 +2247,39 @@ dump_ps_page_header (char *fname, int empty)
   else
     OUTPUT ((cofp, "/user_header_p false def\n"));
 
+  /* User supplied footer? */
+  if (page_footer)
+    {
+      char *f_left;
+      char *f_center;
+      char *f_right = NULL;
+
+      f_left = format_user_string ("page footer", page_footer);
+      f_center = strchr (f_left, '|');
+      if (f_center)
+	{
+	  *f_center = '\0';
+	  f_center++;
+
+	  f_right = strchr (f_center, '|');
+	  if (f_right)
+	    {
+	      *f_right = '\0';
+	      f_right++;
+	    }
+	}
+
+      OUTPUT ((cofp, "/user_footer_p true def\n"));
+      OUTPUT ((cofp, "/user_footer_left_str (%s) def\n", f_left));
+      OUTPUT ((cofp, "/user_footer_center_str (%s) def\n",
+	       f_center ? f_center : ""));
+      OUTPUT ((cofp, "/user_footer_right_str (%s) def\n",
+	       f_right ? f_right : ""));
+      xfree (f_left);
+    }
+  else
+    OUTPUT ((cofp, "/user_footer_p false def\n"));
+
   OUTPUT ((cofp, "%%%%EndPageSetup\n"));
 
   /*
@@ -2274,7 +2377,6 @@ static int
 recognize_eps_file (Token *token)
 {
   int i;
-  char filename[512];
   char buf[4096];
   int line;
   int valid_epsf;
@@ -2299,10 +2401,14 @@ recognize_eps_file (Token *token)
     }
   else
     {
+      char *filename;
+
       /* Read EPS data from file. */
-      tilde_subst (token->u.epsf.filename, filename);
+      filename = tilde_subst (token->u.epsf.filename);
 
       token->u.epsf.fp = fopen (filename, "rb");
+      xfree (filename);
+
       if (token->u.epsf.fp == NULL)
 	{
 	  if (token->u.epsf.filename[0] != '/')
@@ -2310,11 +2416,14 @@ recognize_eps_file (Token *token)
 	      /* Name is not absolute, let's lookup path. */
 	      FileLookupCtx ctx;
 
-	      strcpy (ctx.name, token->u.epsf.filename);
-	      strcpy (ctx.suffix, "");
+	      ctx.name = token->u.epsf.filename;
+	      ctx.suffix = "";
+	      ctx.fullname = buffer_alloc ();
 
 	      if (pathwalk (libpath, file_lookup, &ctx))
-		token->u.epsf.fp = fopen (ctx.fullname, "rb");
+		token->u.epsf.fp = fopen (buffer_ptr (ctx.fullname), "rb");
+
+	      buffer_free (ctx.fullname);
 	    }
 	  if (token->u.epsf.fp == NULL)
 	    {
@@ -2696,31 +2805,15 @@ static char divertfname[512];
 static void
 divert ()
 {
-  char *cp;
-
   assert (divertfp == NULL);
 
   /* Open divert file. */
 
-  cp = tempnam (NULL, "ens");
-  if (cp == NULL)
-    FATAL ((stderr, _("couldn't create divert file name: %s"),
-	    strerror (errno)));
-
-  strcpy (divertfname, cp);
-
-  divertfp = fopen (divertfname, "w+b");
+  divertfp = tmpfile ();
   if (divertfp == NULL)
-    FATAL ((stderr, _("couldn't create divert file \"%s\": %s"), divertfname,
+    FATAL ((stderr, _("couldn't create temporary divert file: %s"),
 	    strerror (errno)));
 
-  if (remove (divertfname) == 0)
-    /* Remove successfull, no need to remove file in undivert(). */
-    divertfname[0] = '\0';
-
-  /* Free the buffer allocated by tempnam(). */
-  free (cp);
-
   cofp = divertfp;
 }
 
@@ -2775,10 +2868,6 @@ undivert ()
   fclose (divertfp);
   divertfp = NULL;
 
-  /* Do we have to remove the divert file? */
-  if (divertfname[0])
-    (void) remove (divertfname);
-
   cofp = ofp;
 }
 
@@ -2790,4 +2879,8 @@ handle_two_side_options ()
     /* Rotate page 180 degrees. */
     OUTPUT ((cofp, "180 rotate\n%d %d translate\n",
 	     -media->w, -media->h));
+
+  if (swap_even_page_margins)
+    OUTPUT ((cofp, "%d 0 translate\n",
+	     -(media->llx - (media->w - media->urx))));
 }
