@@ -1,6 +1,6 @@
 /*
  * General helper utilities.
- * Copyright (c) 1997-1998 Markku Rossi.
+ * Copyright (c) 1997-1999 Markku Rossi.
  *
  * Author: Markku Rossi <mtr@iki.fi>
  */
@@ -525,6 +525,11 @@ lookup_var (env, ns, sym, filename, linenum)
 	      n->u.str.len = len;
 	    }
 
+	  /* Must set the refcount to 0 so that the user will free it
+             it when it is not needed anymore.  We will never touch
+             this node after this pointer. */
+	  n->refcount = 0;
+
 	  return n;
 	}
 
@@ -545,6 +550,8 @@ lookup_var (env, ns, sym, filename, linenum)
 	      memcpy (n->u.str.data, current_match_buf, n->u.str.len);
 	    }
 
+	  /* Set the refcount to 0.  See above. */
+	  n->refcount = 0;
 	  return n;
 	}
 
@@ -553,6 +560,9 @@ lookup_var (env, ns, sym, filename, linenum)
 	{
 	  n = node_alloc (nINTEGER);
 	  n->u.integer = current_linenum;
+
+	  /* Set the refcount to 0.  See above. */
+	  n->refcount = 0;
 	  return n;
 	}
     }
@@ -853,14 +863,16 @@ eval_expr (expr, env)
 	  /* Check that we had correct amount of arguments. */
 	  if (i)
 	    {
-	      fprintf (stderr, _("%s: too few arguments for subroutine\n"),
-		       program);
+	      fprintf (stderr,
+		       _("%s:%d: error: too few arguments for subroutine\n"),
+		       expr->filename, expr->linenum);
 	      exit (1);
 	    }
 	  if (e)
 	    {
-	      fprintf (stderr, _("%s: too many arguments for subroutine\n"),
-		       program);
+	      fprintf (stderr,
+		       _("%s:%d: error: too many arguments for subroutine\n"),
+		       expr->filename, expr->linenum);
 	      exit (1);
 	    }
 
@@ -912,8 +924,9 @@ eval_expr (expr, env)
 	}
       else
 	{
-	  fprintf (stderr, _("%s: undefined procedure `%s'\n"),
-		   program, n->u.sym);
+	  fprintf (stderr,
+		   _("%s:%d: error: undefined procedure `%s'\n"),
+		   expr->filename, expr->linenum, n->u.sym);
 	  exit (1);
 	}
       break;
